# -*- coding: utf-8 -*-
"""
/***************************************************************************
 GNSS_Logger_Plugin
                                 A QGIS plugin
        begin                : 2018-03-19
        copyright            : (C) 2018 by n-survey
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""
from PyQt4.QtCore import *
from PyQt4.QtGui import *
from qgis.core import *
from qgis.gui import *
from qgis import core, utils, gui
import qgis.utils
import os
import sys
import math
import codecs
import datetime
import serial
import threading
import time
import webbrowser
from ui_dialog import Ui_dialog
import re


class GNSS_Logger:
    def __init__(self, iface):
        self.iface = iface

    def initGui(self):
        self.action1 = QAction(u'ウィンドウ位置初期化', self.iface.mainWindow())
        self.iface.addPluginToMenu('GNSS_Logger_Plugin',self.action1)
        self.action1.triggered.connect(self.run2)
        #icon = os.path.dirname(__file__) + u'/GPS2.png'
        icon = ':/icon/GPS.png'
        self.action = QAction(QIcon(icon),'GNSS_Logger_Plugin', self.iface.mainWindow())
        self.iface.addPluginToMenu('GNSS_Logger_Plugin',self.action)
        self.iface.addToolBarIcon(self.action)
        self.action.triggered.connect(self.run)
        self.mapTool = CaptureCoordinates(self.iface)

    def unload(self):
        self.iface.removeToolBarIcon(self.action)
        self.iface.removePluginMenu('GNSS_Logger_Plugin',self.action1)
        self.iface.removePluginMenu('GNSS_Logger_Plugin',self.action)

    def run(self):
        self.iface.mapCanvas().setMapTool(self.mapTool)
        if not hasattr(self, 'dlg'):
            self.dlg = GNSS_LoggerDialog()
            # ウィンドウタイトル表示（バージョン取得）
            for line in codecs.open(os.path.dirname(__file__) + u'/metadata.txt', 'r', 'utf-8'):
                if line[0:8] == 'version=':
                    version = line.replace('version=', '')
            self.dlg.setWindowTitle('GNSS_Logger_Plugin Ver.' + version)
        self.dlg.show()
        # ログファイル保存フォルダ作成
        if not os.path.exists(os.path.dirname(__file__) + '/log'):
            try:
                os.mkdir(os.path.dirname(__file__) + '/log')
            except:
                pass


    def run2(self):
        # ウィンドウ位置初期化
        ws = QSettings("M'z Room", "GNSS_Logger_Plugin")
        ws.setValue("setting/window_w", 800)
        ws.setValue("setting/window_h", 400)
        ws.setValue("setting/window_x", 8)
        ws.setValue("setting/window_y", 31)



# マップキャンバスクリック時にクリックした地点の座標をクリップボードに送る
class CaptureCoordinates(QgsMapTool):
    def __init__(self, iface):
        QgsMapTool.__init__(self, iface.mapCanvas())
        self.iface = iface


    def canvasReleaseEvent(self, e):
        mPos = self.toMapCoordinates(e.pos())
        y1 = str(mPos.x())
        x1 = str(mPos.y())
        rect = QgsRectangle(float(y1),float(x1),float(y1),float(x1))
        map.setExtent(rect)
        map.refresh()

        # プロジェクトのCRSを取得
        epsgGet()
        if epsgcode != '':
            code = epsgcode
            code = 'epsgcode,' + code
        else:
            code = usercode
            code = 'usercode,' + code
        # クリップボードに送る
        coordinate = 'center_coordinates,' + code + ',' + x1 + ',' + y1
        clipboard = QApplication.clipboard()
        clipboard.setText(coordinate)



class GNSS_LoggerDialog(QDialog):
    def __init__(self):
        QDialog.__init__(self)
        self.ui = Ui_dialog()
        self.ui.setupUi(self)
        global map
        map = qgis.utils.iface.mapCanvas()

        ws = QSettings("M'z Room", "GNSS_Logger_Plugin")
        window_x = ws.value("setting/window_x", 8)
        window_y = ws.value("setting/window_y", 32)
        window_w = ws.value("setting/window_w", 800)
        window_h = ws.value("setting/window_h", 400)
        self.setGeometry(window_x, window_y, window_w, window_h)

        self.ui.kei.addItem(u'世界測地系 (JGD2011)  1 系　： EPSG:6669')
        self.ui.kei.addItem(u'世界測地系 (JGD2011)  2 系　： EPSG:6670')
        self.ui.kei.addItem(u'世界測地系 (JGD2011)  3 系　： EPSG:6671')
        self.ui.kei.addItem(u'世界測地系 (JGD2011)  4 系　： EPSG:6672')
        self.ui.kei.addItem(u'世界測地系 (JGD2011)  5 系　： EPSG:6673')
        self.ui.kei.addItem(u'世界測地系 (JGD2011)  6 系　： EPSG:6674')
        self.ui.kei.addItem(u'世界測地系 (JGD2011)  7 系　： EPSG:6675')
        self.ui.kei.addItem(u'世界測地系 (JGD2011)  8 系　： EPSG:6676')
        self.ui.kei.addItem(u'世界測地系 (JGD2011)  9 系　： EPSG:6677')
        self.ui.kei.addItem(u'世界測地系 (JGD2011) 10 系　： EPSG:6678')
        self.ui.kei.addItem(u'世界測地系 (JGD2011) 11 系　： EPSG:6679')
        self.ui.kei.addItem(u'世界測地系 (JGD2011) 12 系　： EPSG:6680')
        self.ui.kei.addItem(u'世界測地系 (JGD2011) 13 系　： EPSG:6681')
        self.ui.kei.addItem(u'世界測地系 (JGD2011) 14 系　： EPSG:6682')
        self.ui.kei.addItem(u'世界測地系 (JGD2011) 15 系　： EPSG:6683')
        self.ui.kei.addItem(u'世界測地系 (JGD2011) 16 系　： EPSG:6684')
        self.ui.kei.addItem(u'世界測地系 (JGD2011) 17 系　： EPSG:6685')
        self.ui.kei.addItem(u'世界測地系 (JGD2011) 18 系　： EPSG:6686')
        self.ui.kei.addItem(u'世界測地系 (JGD2011) 19 系　： EPSG:6687')
        self.ui.kei.setCurrentIndex(12)

        self.ui.comboBox_port.addItem(u'COM1')
        self.ui.comboBox_port.addItem(u'COM2')
        self.ui.comboBox_port.addItem(u'COM3')
        self.ui.comboBox_port.addItem(u'COM4')
        self.ui.comboBox_port.addItem(u'COM5')
        self.ui.comboBox_port.addItem(u'COM6')
        self.ui.comboBox_port.addItem(u'COM7')
        self.ui.comboBox_port.addItem(u'COM8')
        self.ui.comboBox_port.addItem(u'COM9')
        self.ui.comboBox_port.addItem(u'COM10')
        self.ui.comboBox_port.addItem(u'COM11')
        self.ui.comboBox_port.addItem(u'COM12')
        self.ui.comboBox_port.addItem(u'ttyUSB0')
        self.ui.comboBox_port.addItem(u'ttyUSB1')
        self.ui.comboBox_port.addItem(u'ttyACM0')
        self.ui.comboBox_port.addItem(u'ttyACM1')
        self.ui.comboBox_port.addItem(u'tty.usbmodem14101')
        self.ui.comboBox_port.addItem(u'tty.usbmodem14201')
        self.ui.comboBox_port.setCurrentIndex(2)

        self.ui.comboBox_baudrate.addItem(u'4800')
        self.ui.comboBox_baudrate.addItem(u'9600')
        self.ui.comboBox_baudrate.addItem(u'14400')
        self.ui.comboBox_baudrate.addItem(u'19200')
        self.ui.comboBox_baudrate.addItem(u'38400')
        self.ui.comboBox_baudrate.addItem(u'57600')
        self.ui.comboBox_baudrate.addItem(u'115200')
        self.ui.comboBox_baudrate.addItem(u'230400')
        self.ui.comboBox_baudrate.addItem(u'460800')
        self.ui.comboBox_baudrate.setCurrentIndex(1)

        self.ui.comboBox_sentence.addItem(u'GNRMC/GPRMC')
        self.ui.comboBox_sentence.addItem(u'GNGGA/GPGGA')
        self.ui.comboBox_sentence.addItem(u'GNGLL/GPGLL')
        self.ui.comboBox_sentence.addItem(u'NMEA')
        self.ui.comboBox_sentence.addItem(u'RTKNAVI_FIX')
        self.ui.comboBox_sentence.addItem(u'RTKNAVI_ALL')
        self.ui.comboBox_sentence.addItem(u'RTKNAVI_RATIO')
        self.ui.comboBox_sentence.setCurrentIndex(1)

        self.ui.comboBox_CRS.addItem(u'世界測地系   1 系　： EPSG:2433')
        self.ui.comboBox_CRS.addItem(u'世界測地系   2 系　： EPSG:2434')
        self.ui.comboBox_CRS.addItem(u'世界測地系   3 系　： EPSG:2435')
        self.ui.comboBox_CRS.addItem(u'世界測地系   4 系　： EPSG:2436')
        self.ui.comboBox_CRS.addItem(u'世界測地系   5 系　： EPSG:2437')
        self.ui.comboBox_CRS.addItem(u'世界測地系   6 系　： EPSG:2438')
        self.ui.comboBox_CRS.addItem(u'世界測地系   7 系　： EPSG:2439')
        self.ui.comboBox_CRS.addItem(u'世界測地系   8 系　： EPSG:2440')
        self.ui.comboBox_CRS.addItem(u'世界測地系   9 系　： EPSG:2441')
        self.ui.comboBox_CRS.addItem(u'世界測地系  10 系　： EPSG:2442')
        self.ui.comboBox_CRS.addItem(u'世界測地系  11 系　： EPSG:2443')
        self.ui.comboBox_CRS.addItem(u'世界測地系  12 系　： EPSG:2444')
        self.ui.comboBox_CRS.addItem(u'世界測地系  13 系　： EPSG:2445')
        self.ui.comboBox_CRS.addItem(u'世界測地系  14 系　： EPSG:2446')
        self.ui.comboBox_CRS.addItem(u'世界測地系  15 系　： EPSG:2447')
        self.ui.comboBox_CRS.addItem(u'世界測地系  16 系　： EPSG:2448')
        self.ui.comboBox_CRS.addItem(u'世界測地系  17 系　： EPSG:2449')
        self.ui.comboBox_CRS.addItem(u'世界測地系  18 系　： EPSG:2450')
        self.ui.comboBox_CRS.addItem(u'世界測地系  19 系　： EPSG:2451')
        self.ui.comboBox_CRS.addItem(u'日本測地系   1 系  ： EPSG:30161')
        self.ui.comboBox_CRS.addItem(u'日本測地系   2 系  ： EPSG:30162')
        self.ui.comboBox_CRS.addItem(u'日本測地系   3 系  ： EPSG:30163')
        self.ui.comboBox_CRS.addItem(u'日本測地系   4 系  ： EPSG:30164')
        self.ui.comboBox_CRS.addItem(u'日本測地系   5 系  ： EPSG:30165')
        self.ui.comboBox_CRS.addItem(u'日本測地系   6 系  ： EPSG:30166')
        self.ui.comboBox_CRS.addItem(u'日本測地系   7 系  ： EPSG:30167')
        self.ui.comboBox_CRS.addItem(u'日本測地系   8 系  ： EPSG:30168')
        self.ui.comboBox_CRS.addItem(u'日本測地系   9 系  ： EPSG:30169')
        self.ui.comboBox_CRS.addItem(u'日本測地系  10 系  ： EPSG:30170')
        self.ui.comboBox_CRS.addItem(u'日本測地系  11 系  ： EPSG:30171')
        self.ui.comboBox_CRS.addItem(u'日本測地系  12 系  ： EPSG:30172')
        self.ui.comboBox_CRS.addItem(u'日本測地系  13 系  ： EPSG:30173')
        self.ui.comboBox_CRS.addItem(u'日本測地系  14 系  ： EPSG:30174')
        self.ui.comboBox_CRS.addItem(u'日本測地系  15 系  ： EPSG:30175')
        self.ui.comboBox_CRS.addItem(u'日本測地系  16 系  ： EPSG:30176')
        self.ui.comboBox_CRS.addItem(u'日本測地系  17 系  ： EPSG:30177')
        self.ui.comboBox_CRS.addItem(u'日本測地系  18 系  ： EPSG:30178')
        self.ui.comboBox_CRS.addItem(u'日本測地系  19 系  ： EPSG:30179')
        self.ui.comboBox_CRS.addItem(u'世界測地系 UTMzone51N ： EPSG:3097')
        self.ui.comboBox_CRS.addItem(u'世界測地系 UTMzone51N ： EPSG:3098')
        self.ui.comboBox_CRS.addItem(u'世界測地系 UTMzone52N ： EPSG:3099')
        self.ui.comboBox_CRS.addItem(u'世界測地系 UTMzone53N ： EPSG:3100')
        self.ui.comboBox_CRS.addItem(u'世界測地系 UTMzone54N ： EPSG:3101')
        self.ui.comboBox_CRS.addItem(u'日本測地系 UTMzone51N ： EPSG:3092')
        self.ui.comboBox_CRS.addItem(u'日本測地系 UTMzone52N ： EPSG:3093')
        self.ui.comboBox_CRS.addItem(u'日本測地系 UTMzone53N ： EPSG:3094')
        self.ui.comboBox_CRS.addItem(u'日本測地系 UTMzone54N ： EPSG:3095')
        self.ui.comboBox_CRS.addItem(u'日本測地系 UTMzone55N ： EPSG:3096')
        self.ui.comboBox_CRS.addItem(u'WGS84 UTMzone51N ： EPSG:32651')
        self.ui.comboBox_CRS.addItem(u'WGS84 UTMzone52N ： EPSG:32652')
        self.ui.comboBox_CRS.addItem(u'WGS84 UTMzone53N ： EPSG:32653')
        self.ui.comboBox_CRS.addItem(u'WGS84 UTMzone54N ： EPSG:32654')
        self.ui.comboBox_CRS.addItem(u'WGS84 UTMzone55N ： EPSG:32655')
        self.ui.comboBox_CRS.addItem(u'WGS 84 ： EPSG:4326')
        self.ui.comboBox_CRS.addItem(u'世界測地系　JGD2000 ： EPSG:4612')
        self.ui.comboBox_CRS.addItem(u'日本測地系  Tokyo　：　EPSG:4301')
        self.ui.comboBox_CRS.addItem(u'WGS 84 / Pseudo Mercator ： EPSG:3857')
        ucfname = os.path.dirname(__file__) + u'/user_crs.txt'
        if os.path.exists(ucfname):
            for line in codecs.open(ucfname,'r', 'shift-jis'):
                self.ui.comboBox_CRS.addItem(line.replace((u'\n' or u'\r\n'),''))
        # プロジェクトのCRSを取得
        epsgGet()
        if epsgcode != '':
            code = epsgcode
        else:
            code = usercode
        epsg = 0
        try:
            for count in range(self.ui.comboBox_CRS.count()):
                dat = self.ui.comboBox_CRS.itemText(count).split(':')
                if int(dat[1]) == int(code):
                    epsg = count
        except:
            pass
        self.ui.comboBox_CRS.setCurrentIndex(epsg)

        self.ui.switch_on.toggled.connect(self.on_off)
        self.ui.clear_log.clicked.connect(self.clearlog)
        self.ui.check_plot.clicked.connect(self.SetEnable2)
        self.ui.check_tansaku.clicked.connect(self.SetEnable3)
        self.ui.check_shousai.clicked.connect(self.SetEnable4)
        self.ui.comboBox_sentence.currentIndexChanged.connect(self.SetEnable5)
        self.ui.GPX.clicked.connect(self.gpx_save)
        self.ui.SIMA.clicked.connect(self.sima_save)
        self.ui.KML.clicked.connect(self.kml_save)
        self.ui.pushButton_CRS.clicked.connect(self.change_CRS)
        self.ui.epoch.clicked.connect(self.changeText)

        gnss_kei = ws.value("setting/GNSS_Kei", 12)                # 平面直角座標系
        gnss_port = ws.value("setting/GNSS_Port", 3)               # Port
        gnss_baudrate = ws.value("setting/GNSS_Baudrate", 8)       # Baudrate
        gnss_sentence = ws.value("setting/GNSS_Sentence", 1)       # Sentence
        gnss_time = ws.value("setting/GNSS_Time", '0')             # Time
        gnss_plot = ws.value("setting/GNSS_Plot", '0')             # プロット
        gnss_centering = ws.value("setting/GNSS_Centering", '0')   # センタリング
        gnss_log = ws.value("setting/GNSS_Log", '0')               # ログ表示
        gnss_view = ws.value("setting/GNSS_View", '0')             # 詳細表示
        self.ui.kei.setCurrentIndex(gnss_kei)
        self.ui.comboBox_port.setCurrentIndex(gnss_port)
        self.ui.comboBox_baudrate.setCurrentIndex(gnss_baudrate)
        self.ui.comboBox_sentence.setCurrentIndex(gnss_sentence)
        if gnss_sentence == 6:
            self.ui.lineEdit_Ratio.setEnabled(True)
        else:
            self.ui.lineEdit_Ratio.setEnabled(False)
        if gnss_time == '1':
            self.ui.radioButton_time1.setChecked(True)
        else:
            self.ui.radioButton_time2.setChecked(True)
        if gnss_plot == '1':
            self.ui.check_plot.setChecked(True)
        else:
            self.ui.check_plot.setChecked(False)
        if gnss_centering == '1':
            self.ui.check_centering.setChecked(True)
        else:
            self.ui.check_centering.setChecked(False)
        if gnss_log == '1':
            self.ui.check_log.setChecked(True)
        else:
            self.ui.check_log.setChecked(False)
        if gnss_view == '1':
            self.ui.check_shousai.setChecked(True)
        else:
            self.ui.check_shousai.setChecked(False)
        if self.ui.check_plot.isChecked():
            self.ui.check_centering.setEnabled(True)
        else:
            self.ui.check_centering.setChecked(False)
            self.ui.check_centering.setEnabled(False)
        self.SetEnable5()
        self.ui.switch_on.setFocus()

    # タブ変更
    def setTab(self):
        if self.ui.tabWidget.currentIndex() == 0:
            # プロジェクトのCRSを取得
            epsgGet()
            if epsgcode != '':
                code = epsgcode
            else:
                code = usercode
            epsg = 0
            try:
                for count in range(self.ui.comboBox_CRS.count()):
                    dat = self.ui.comboBox_CRS.itemText(count).split(':')
                    if int(dat[1]) == int(code):
                        epsg = count
            except:
                pass
            self.ui.comboBox_CRS.setCurrentIndex(epsg)

    # epochボタン名変更
    def changeText(self):
        self.ui.label_x2.clear()
        self.ui.label_y2.clear()
        self.ui.label_h2.clear()
        if self.ui.epoch.isChecked():
            self.ui.epoch.setText(u'取得中')
        else:
            self.ui.epoch.setText(u'開始')

    # check_plotクリック
    def SetEnable2(self):
        if self.ui.check_plot.isChecked():
            self.ui.check_centering.setEnabled(True)
        else:
            self.ui.check_centering.setChecked(False)
            self.ui.check_centering.setEnabled(False)

    # check_tansakuクリック
    def SetEnable3(self):
        if self.ui.check_tansaku.isChecked():
            self.ui.x1.setEnabled(True)
            self.ui.y1.setEnabled(True)
            self.ui.a.setEnabled(True)
            self.get_center()
            self.ui.x1.setFocus()
        else:
            self.ui.x1.clear()
            self.ui.y1.clear()
            self.ui.a.clear()
            self.ui.x1.setEnabled(False)
            self.ui.y1.setEnabled(False)
            self.ui.a.setEnabled(False)
            clipboard = QApplication.clipboard()
            clipboard.clear()

    # 詳細クリア
    def SetEnable4(self):
        self.ui.label_x.clear()
        self.ui.label_y.clear()
        self.ui.label_h.clear()
        self.ui.label_ratio.clear()

    # Sentexce選択
    def SetEnable5(self):
        if self.ui.comboBox_sentence.currentText() == 'NMEA':
            self.ui.check_convert2.setChecked(False)
            self.ui.check_plot.setChecked(False)
            self.ui.check_centering.setChecked(False)
            self.ui.check_tansaku.setChecked(False)
            self.ui.check_shousai.setChecked(False)
            self.ui.check_convert2.setEnabled(False)
            self.ui.check_plot.setEnabled(False)
            self.ui.check_centering.setEnabled(False)
            self.ui.check_tansaku.setEnabled(False)
            #self.ui.check_log.setChecked(False) #QGIS3.xのみ
            #self.ui.check_log.setEnabled(False) #QGIS3.xのみ
        else:
            self.ui.check_convert2.setEnabled(True)
            self.ui.check_plot.setEnabled(True)
            self.ui.check_tansaku.setEnabled(True)
            #self.ui.check_log.setEnabled(True) #QGIS3.xのみ
            if self.ui.comboBox_sentence.currentText() == 'RTKNAVI_RATIO':
                self.ui.lineEdit_Ratio.setEnabled(True)
                self.ui.lineEdit_Ratio.setFocus()
            if self.ui.comboBox_sentence.currentText() == 'RTKNAVI_FIX' or self.ui.comboBox_sentence.currentText() == 'RTKNAVI_ALL' or self.ui.comboBox_sentence.currentText() == 'RTKNAVI_RATIO':
                self.ui.epoch.setEnabled(True)
                self.ui.lineEdit_epoch.setEnabled(True)
        self.SetEnable2()
        self.SetEnable3()
        self.SetEnable4()

    # サポートページを開く
    def open_browser(self):
        kakunin = QMessageBox.question(self,u'サポートページ', u'サポートページをブラウザで開きます。\n実行してもよろしいですか？',QMessageBox.Yes | QMessageBox.No,QMessageBox.No)
        if kakunin == QMessageBox.Yes:
            url = "https://www.n-survey.com/"
            webbrowser.open(url)

    # クリックした地点の平面直角座標をセット
    def get_center(self):
        try:
            clipboard = QApplication.clipboard()
            coordinate = clipboard.text()
            coordinate = coordinate.split(',')
            if coordinate[0] == 'center_coordinates':
                self.ui.x1.setText('')
                self.ui.y1.setText('')
                EPSG1 = int(coordinate[2])
                epsg2 = self.ui.kei.currentText().split(':')
                EPSG2 = int(epsg2[1])
                if EPSG1 == EPSG2:
                    self.ui.x1.setText('{:.3f}'.format(round(float(coordinate[3]), 3)))
                    self.ui.y1.setText('{:.3f}'.format(round(float(coordinate[4]), 3)))
                else:
                    if coordinate[1] == 'usercode':#カスタムCRSの場合
                        Src_crs = QgsCoordinateReferenceSystem()
                        Src_crs.createFromId(EPSG1, QgsCoordinateReferenceSystem.InternalCrsId)
                        crsSrc = QgsCoordinateReferenceSystem(Src_crs)
                    else:
                        Src_crs = QgsCoordinateReferenceSystem(EPSG1)
                        crsSrc = QgsCoordinateReferenceSystem(Src_crs)
                    crsDest = QgsCoordinateReferenceSystem(EPSG2)
                    trans = QgsCoordinateTransform(crsSrc, crsDest)
                    y2,x2 = trans.transform(QgsPoint(float(coordinate[4]), float(coordinate[3])))
                    self.ui.x1.setText('{:.3f}'.format(round(x2, 3)))
                    self.ui.y1.setText('{:.3f}'.format(round(y2, 3)))
        except:
            pass

    # 消去
    def clearlog(self):
        kakunin = QMessageBox.question(self,u'消去', u'表示中のログを消去します。\n実行してもよろしいですか？',QMessageBox.Yes | QMessageBox.No,QMessageBox.No)
        if kakunin == QMessageBox.Yes:
            self.ui.textLog.clear()
            self.ui.x1.clear()
            self.ui.y1.clear()
            self.ui.a.clear()
            self.ui.label_x.clear()
            self.ui.label_y.clear()
            self.ui.label_h.clear()
            self.ui.label_ratio.clear()
            self.ui.label_x2.setText('')
            self.ui.label_y2.setText('')
            self.ui.label_h2.setText('')
            self.ui.check_tansaku.setChecked(False)

    # 接続・切断
    def on_off(self, checked):
        self.stop_event = threading.Event()
        self.inc_event = threading.Event()
        self.thread = threading.Thread(target = self.get_log)
        global oldProjValue
        if checked:
            if self.ui.check_tansaku.isChecked():
                if self.ui.x1.text() == '':
                    QMessageBox.information(self, u'エラー', u'探索する測点のX座標を設定してください。')
                    self.ui.switch_on.setChecked(False)
                    return
                if self.ui.y1.text() == '':
                    QMessageBox.information(self, u'エラー', u'探索する測点のY座標を設定してください。')
                    self.ui.switch_on.setChecked(False)
                    return
            if self.ui.textLog.toPlainText() != '':
                kakunin = QMessageBox.question(self,u'受信', u'表示中のログは消去されます。\n実行してもよろしいですか？',QMessageBox.Yes | QMessageBox.No,QMessageBox.No)
                if kakunin == QMessageBox.No:
                    self.ui.switch_on.setChecked(False)
                    return
            # オンザフライ投影変換を有効にする
            qgis.utils.iface.mapCanvas().mapRenderer().setProjectionsEnabled(True)
            # 開始時のCRS設定を記憶する
            oldProjValue = QSettings().value('/Projections/defaultBehaviour', 'prompt', type = str)
            # プロジェクトのCRSを使用するモードに設定する
            QSettings().setValue('/Projections/defaultBehaviour', 'useProject')
            self.ui.switch_on.setText(u'切断')
            self.ui.GPX.setEnabled(False)
            self.ui.SIMA.setEnabled(False)
            self.ui.KML.setEnabled(False)
            self.ui.clear_log.setEnabled(False)
            self.ui.kei.setEnabled(False)
            self.ui.check_convert2.setEnabled(False)
            self.ui.radioButton_time1.setEnabled(False)
            self.ui.radioButton_time2.setEnabled(False)
            self.ui.comboBox_port.setEnabled(False)
            self.ui.comboBox_baudrate.setEnabled(False)
            self.ui.comboBox_sentence.setEnabled(False)
            self.ui.comboBox_CRS.setEnabled(False)
            self.ui.pushButton_CRS.setEnabled(False)
            self.ui.label_1.setEnabled(False)
            self.ui.label_2.setEnabled(False)
            self.ui.label_3.setEnabled(False)
            self.ui.label_4.setEnabled(False)
            self.ui.textLog.clear()
            self.ui.label_x.clear()
            self.ui.label_y.clear()
            self.ui.label_h.clear()
            self.ui.label_ratio.clear()
            if not self.ui.check_log.isChecked():
                self.ui.textLog.append(u'ログ記録中')
            self.thread.start()
        else:
            self.ui.switch_on.setText(u'受信開始')
            self.ui.GPX.setEnabled(True)
            self.ui.SIMA.setEnabled(True)
            self.ui.KML.setEnabled(True)
            self.ui.clear_log.setEnabled(True)
            self.ui.kei.setEnabled(True)
            self.ui.check_convert2.setEnabled(True)
            self.ui.check_tansaku.setEnabled(True)
            self.ui.check_plot.setEnabled(True)
            self.ui.comboBox_port.setEnabled(True)
            self.ui.comboBox_baudrate.setEnabled(True)
            self.ui.comboBox_sentence.setEnabled(True)
            self.ui.radioButton_time1.setEnabled(True)
            self.ui.radioButton_time2.setEnabled(True)
            self.ui.comboBox_CRS.setEnabled(True)
            self.ui.pushButton_CRS.setEnabled(True)
            self.ui.label_1.setEnabled(True)
            self.ui.label_2.setEnabled(True)
            self.ui.label_3.setEnabled(True)
            self.ui.label_4.setEnabled(True)
            self.ui.label_14.setEnabled(True)
            if self.ui.check_plot.isChecked():
                self.ui.check_centering.setEnabled(True)
            else:
                self.ui.check_centering.setChecked(False)
                self.ui.check_centering.setEnabled(False)
            if not self.ui.check_log.isChecked():
                self.ui.textLog.clear()
            self.ui.epoch.setChecked(False)
            self.ui.epoch.setText(u'開始')
            self.stop_event.set()
            # プロジェクトのCRS設定を開始時に戻す
            QSettings().setValue('/Projections/defaultBehaviour', str(oldProjValue))

    def stop(self):
        self.stop_event.set()
        self.thread.join()

    # 受信
    def get_log(self):
        global daytime
        global b
        global l
        global h
        global g
        global x1
        global y1
        global x2
        global y2
        global x01
        global y01
        global x02
        global y02
        global EPSG2
        global EPSG3
        x0 = 999999
        y0 = 999999
        epsg2 = self.ui.kei.currentText().split(':')
        EPSG2 = int(epsg2[1])
        epsgGet()
        if epsgcode != '':
            EPSG3 = int(epsgcode)
        else:
            EPSG3 = int(usercode)

        if os.name == 'nt':
            ser = serial.Serial(
                port=self.ui.comboBox_port.currentText(),
                baudrate=int(self.ui.comboBox_baudrate.currentText()),
                bytesize=8,
                parity='N',
                stopbits=1,
                timeout=1)#,
                #xonxoff=0,
                #rtscts=0,
                #writeTimeout=None,
                #dsrdtr=None)
        else:
            ser = serial.Serial(
                port='/dev/' + self.ui.comboBox_port.currentText(),\
                baudrate=int(self.ui.comboBox_baudrate.currentText()),\
                parity=serial.PARITY_NONE,\
                stopbits=serial.STOPBITS_ONE,\
                bytesize=serial.EIGHTBITS,\
                    timeout=1)

        def degrees_to_decimal(data, hemisphere):
            try:
                decimalPointPosition = data.index('.')
                degrees = float(data[:decimalPointPosition-2])
                minutes = float(data[decimalPointPosition-2:])/60
                output = degrees + minutes
                if hemisphere is 'N' or hemisphere is 'E':
                    return output
                if hemisphere is 'S' or hemisphere is 'W':
                    return -output
            except:
                return ""

        def parse_GNRMC(data):
            data = data.split(',')
            dict = {
                    'fix_time': data[1],
                    'Status': data[2],
                    'latitude': data[3],
                    'NothingIndicator': data[4],
                    'longitude': data[5],
                    'EastingIndicator': data[6],
                    'speed': data[7],
                    'true_course': data[8],
                    'fix_date': data[9],
                    'variation': data[10],
                    'variation_e_w': data[11],
                    'checksum': data[12]
            }
            dict['decimal_latitude'] = degrees_to_decimal(dict['latitude'], dict['NothingIndicator'])
            dict['decimal_longitude'] = degrees_to_decimal(dict['longitude'], dict['EastingIndicator'])
            return dict

        def parse_GNGGA(data):
            data = data.split(',')
            dict = {
                    'fix_time': data[1],
                    'latitude': data[2],
                    'NothingIndicator': data[3],
                    'longitude': data[4],
                    'EastingIndicator': data[5],
                    'Status': data[6],
                    'SVsUsed': data[7],
                    'HDOP': data[8],
                    'altitude': data[9],
                    'unit1': data[10],
                    'geoid': data[11],
                    'unit2': data[12],
                    'dgps': data[13],
                    'checksum': data[14]
            }
            dict['decimal_latitude'] = degrees_to_decimal(dict['latitude'], dict['NothingIndicator'])
            dict['decimal_longitude'] = degrees_to_decimal(dict['longitude'], dict['EastingIndicator'])
            return dict

        def parse_GNGLL(data):
            data = data.split(',')
            dict = {
                    'latitude': data[1],
                    'NothingIndicator': data[2],
                    'longitude': data[3],
                    'EastingIndicator': data[4],
                    'fix_time': data[5],
                    'Status': data[6],
                    'ModeIndicator': data[7]
            }
            dict['decimal_latitude'] = degrees_to_decimal(dict['latitude'], dict['NothingIndicator'])
            dict['decimal_longitude'] = degrees_to_decimal(dict['longitude'], dict['EastingIndicator'])
            return dict

        startTime = str(datetime.datetime.now())[0:19].replace('-', '').replace(':', '').replace(' ', '')

        if self.ui.comboBox_sentence.currentText() == 'GNRMC/GPRMC':

            while not self.stop_event.is_set():
                line = ser.readline()
                if '$GNRMC' in line or '$GPRMC' in line:
                    if '$GNRMC' in line:
                        gn = line.split('$GNRMC')
                        line = '$GNRMC' + gn[1]
                        log_name = 'GNRMC'
                    else:
                        gn = line.split('$GPRMC')
                        line = '$GPRMC' + gn[1]
                        log_name = 'GPRMC'
                    gpsData = parse_GNRMC(line)
                    if gpsData['Status'] == 'A':
                        day_time = datetime.datetime.now()
                        if self.ui.radioButton_time1.isChecked():
                            daytime = str(day_time)[0:19].replace('-', '').replace(':', '').replace(' ', '')
                        else:
                            daytime = str(gpsData['fix_date']) + str(gpsData['fix_time'])
                        b = '{:.9f}'.format(round(float(gpsData['decimal_latitude']), 9))
                        l = '{:.9f}'.format(round(float(gpsData['decimal_longitude']), 9))
                        h = ''
                        g = ''
                        crsSrc = QgsCoordinateReferenceSystem(4326)
                        crsDest = QgsCoordinateReferenceSystem(EPSG2)
                        trans = QgsCoordinateTransform(crsSrc, crsDest)
                        y1,x1 = trans.transform(QgsPoint(float(l), float(b)))
                        x01 = '{:.7f}'.format(round(x1, 7))
                        y01 = '{:.7f}'.format(round(y1, 7))
                        if self.ui.check_convert2.isChecked():
                            if custom == '':#カスタムCRSの場合
                                Dest_crs = QgsCoordinateReferenceSystem()
                                Dest_crs.createFromId(EPSG3, QgsCoordinateReferenceSystem.InternalCrsId)
                                crsDest = QgsCoordinateReferenceSystem(Dest_crs)
                            else:
                                crsDest = QgsCoordinateReferenceSystem(EPSG3)
                            trans = QgsCoordinateTransform(crsSrc, crsDest)
                            y2,x2 = trans.transform(QgsPoint(float(l), float(b)))
                            x02 = '{:.7f}'.format(round(x2, 7))
                            y02 = '{:.7f}'.format(round(y2, 7))
                        else:
                            pass
                            x2 = ''
                            y2 = ''
                            x02 = ''
                            y02 = ''
                            EPSG3 = ''
                        log_line = daytime + ',' + b + ',' + l + ',' + h + ',' + g + ',' + str(EPSG2) + ',' + x01 + ',' + y01 + ',' + str(EPSG3) + ',' + x02 + ',' + y02 + ',,'
                        if self.ui.check_tansaku.isChecked():
                            self.tansaku()
                        if self.ui.check_plot.isChecked():
                            self.plot()
                        if self.ui.check_shousai.isChecked():
                            try:
                                self.ui.label_x.setText('{:.3f}'.format(round(x1, 3)))
                                self.ui.label_y.setText('{:.3f}'.format(round(y1, 3)))
                            except:
                                pass
                        if self.ui.check_log.isChecked():
                            self.ui.textLog.append(log_line)
                        with open(os.path.dirname(__file__) + '/log/' + startTime + u'_' + log_name + u".log", "a") as myfile:
                            myfile.write(log_line + '\n')
            ser.close()

        elif self.ui.comboBox_sentence.currentText() == 'GNGGA/GPGGA':

            while not self.stop_event.is_set():
                line = ser.readline()
                if '$GNGGA' in line or '$GPGGA' in line:
                    if '$GNGGA' in line:
                        gn = line.split('$GNGGA')
                        line = '$GNGGA' + gn[1]
                        log_name = 'GNGGA'
                    else:
                        gn = line.split('$GPGGA')
                        line = '$GPGGA' + gn[1]
                        log_name = 'GPGGA'
                    gpsData = parse_GNGGA(line)
                    if gpsData['Status'] == '1' or gpsData['Status'] == '2' or gpsData['Status'] == '4' or gpsData['Status'] == '5':
                        day_time = datetime.datetime.now()
                        if self.ui.radioButton_time1.isChecked():
                            daytime = str(day_time)[0:19].replace('-', '').replace(':', '').replace(' ', '')
                        else:
                            daytime = str(gpsData['fix_time'])
                        b = '{:.9f}'.format(round(float(gpsData['decimal_latitude']), 9))
                        l = '{:.9f}'.format(round(float(gpsData['decimal_longitude']), 90))
                        h = str(gpsData['altitude'])
                        g = str(gpsData['geoid'])
                        crsSrc = QgsCoordinateReferenceSystem(4326)
                        crsDest = QgsCoordinateReferenceSystem(EPSG2)
                        trans = QgsCoordinateTransform(crsSrc, crsDest)
                        y1,x1 = trans.transform(QgsPoint(float(l), float(b)))
                        x01 = '{:.7f}'.format(round(x1, 7))
                        y01 = '{:.7f}'.format(round(y1, 7))
                        if self.ui.check_convert2.isChecked():
                            if custom == '':#カスタムCRSの場合
                                Dest_crs = QgsCoordinateReferenceSystem()
                                Dest_crs.createFromId(EPSG3, QgsCoordinateReferenceSystem.InternalCrsId)
                                crsDest = QgsCoordinateReferenceSystem(Dest_crs)
                            else:
                                crsDest = QgsCoordinateReferenceSystem(EPSG3)
                            trans = QgsCoordinateTransform(crsSrc, crsDest)
                            y2,x2 = trans.transform(QgsPoint(float(l), float(b)))
                            x02 = '{:.7f}'.format(round(x2, 7))
                            y02 = '{:.7f}'.format(round(y2, 7))
                        else:
                            pass
                            x2 = ''
                            y2 = ''
                            x02 = ''
                            y02 = ''
                            EPSG3 = ''
                        log_line = daytime + ',' + b + ',' + l + ',' + h + ',' + g + ',' + str(EPSG2) + ',' + x01 + ',' + y01 + ',' + str(EPSG3) + ',' + x02 + ',' + y02 + ',,'
                        if self.ui.check_tansaku.isChecked():
                            self.tansaku()
                        if self.ui.check_plot.isChecked():
                            self.plot()
                        if self.ui.check_shousai.isChecked():
                            try:
                                self.ui.label_x.setText('{:.3f}'.format(round(x1, 3)))
                                self.ui.label_y.setText('{:.3f}'.format(round(y1, 3)))
                                self.ui.label_h.setText(h)
                            except:
                                pass
                        if self.ui.check_log.isChecked():
                            self.ui.textLog.append(log_line)
                        with open(os.path.dirname(__file__) + '/log/' + startTime + u'_' + log_name + u".log", "a") as myfile:
                            myfile.write(log_line + '\n')
            ser.close()

        elif self.ui.comboBox_sentence.currentText() == 'GNGLL/GPGLL':

            while not self.stop_event.is_set():
                line = ser.readline()
                if '$GNGLL' in line or '$GPGLL' in line:
                    if '$GNGLL' in line:
                        gn = line.split('$GNGLL')
                        line = '$GNGLL' + gn[1]
                        log_name = 'GNGLL'
                    else:
                        gn = line.split('$GPGLL')
                        line = '$GPGLL' + gn[1]
                        log_name = 'GPGLL'
                    gpsData = parse_GNGLL(line)
                    if gpsData['Status'] == 'A':
                        day_time = datetime.datetime.now()
                        if self.ui.radioButton_time1.isChecked():
                            daytime = str(day_time)[0:19].replace('-', '').replace(':', '').replace(' ', '')
                        else:
                            daytime = str(gpsData['fix_time'])
                        b = '{:.9f}'.format(round(float(gpsData['decimal_latitude']), 9))
                        l = '{:.9f}'.format(round(float(gpsData['decimal_longitude']), 9))
                        h = ''
                        g = ''
                        crsSrc = QgsCoordinateReferenceSystem(4326)
                        crsDest = QgsCoordinateReferenceSystem(EPSG2)
                        trans = QgsCoordinateTransform(crsSrc, crsDest)
                        y1,x1 = trans.transform(QgsPoint(float(l), float(b)))
                        x01 = '{:.7f}'.format(round(x1, 7))
                        y01 = '{:.7f}'.format(round(y1, 7))
                        if self.ui.check_convert2.isChecked():
                            if custom == '':#カスタムCRSの場合
                                Dest_crs = QgsCoordinateReferenceSystem()
                                Dest_crs.createFromId(EPSG3, QgsCoordinateReferenceSystem.InternalCrsId)
                                crsDest = QgsCoordinateReferenceSystem(Dest_crs)
                            else:
                                crsDest = QgsCoordinateReferenceSystem(EPSG3)
                            trans = QgsCoordinateTransform(crsSrc, crsDest)
                            y2,x2 = trans.transform(QgsPoint(float(l), float(b)))
                            x02 = '{:.7f}'.format(round(x2, 7))
                            y02 = '{:.7f}'.format(round(y2, 7))
                        else:
                            pass
                            x2 = ''
                            y2 = ''
                            x02 = ''
                            y02 = ''
                            EPSG3 = ''
                        log_line = daytime + ',' + b + ',' + l + ',' + h + ',' + g + ',' + str(EPSG2) + ',' + x01 + ',' + y01 + ',' + str(EPSG3) + ',' + x02 + ',' + y02 + ',,'
                        if self.ui.check_tansaku.isChecked():
                            self.tansaku()
                        if self.ui.check_plot.isChecked():
                            self.plot()
                        if self.ui.check_shousai.isChecked():
                            try:
                                self.ui.label_x.setText('{:.3f}'.format(round(x1, 3)))
                                self.ui.label_y.setText('{:.3f}'.format(round(y1, 3)))
                            except:
                                pass
                        if self.ui.check_log.isChecked():
                            self.ui.textLog.append(log_line)
                        with open(os.path.dirname(__file__) + '/log/' + startTime + u'_' + log_name + u".log", "a") as myfile:
                            myfile.write(log_line + '\n')
            ser.close()

        elif self.ui.comboBox_sentence.currentText() == 'NMEA':

            while not self.stop_event.is_set():
                line = ser.readline()
                line = line.replace('\r\n', '')
                if line[0:1] != '$':
                    if '$GNRMC' in line:
                        gn = line.split('$GNRMC')
                        line = '$GNRMC' + gn[1]
                    if '$GNGGA' in line:
                        gn = line.split('$GNGGA')
                        line = '$GNGGA' + gn[1]
                    if '$GNGLL' in line:
                        gn = line.split('$GNGLL')
                        line = '$GNGLL' + gn[1]
                    if '$GPRMC' in line:
                        gn = line.split('$GPRMC')
                        line = '$GPRMC' + gn[1]
                    if '$GPGGA' in line:
                        gn = line.split('$GPGGA')
                        line = '$GPGGA' + gn[1]
                    if '$GPGLL' in line:
                        gn = line.split('$GPGLL')
                        line = '$GPGLL' + gn[1]
                if line[0:1] == '$':
                    if self.ui.check_log.isChecked():
                        self.ui.textLog.append(str(line))
                    with open(os.path.dirname(__file__) + '/log/' + startTime + u"_NMEA.log", "a") as myfile:
                        myfile.write(line)
            ser.close()

        else:

            log_name = self.ui.comboBox_sentence.currentText()
            n = 0
            x001 = 0
            y001 = 0
            h001 = 0

            while not self.stop_event.is_set():
                try:
                    line = ser.readline()
                    line = re.sub(r"\s+", ",", line).replace('\r\n', '')
                    if line != '':
                        dat = line.split(',')
                        if (self.ui.comboBox_sentence.currentText() == 'RTKNAVI_FIX' and dat[5] == '1') or (self.ui.comboBox_sentence.currentText() == 'RTKNAVI_ALL' and (dat[5] == '1' or dat[5] == '2')) or (self.ui.comboBox_sentence.currentText() == 'RTKNAVI_RATIO' and float(dat[14]) >= float(self.ui.lineEdit_Ratio.text())):
                            day_time = datetime.datetime.now()
                            if self.ui.radioButton_time1.isChecked():
                                daytime = str(day_time)[0:19].replace('-', '').replace(':', '').replace(' ', '')
                            else:
                                daytime = dat[1].replace(':', '')
                            b = '{:.9f}'.format(round(float(dat[2]), 9))
                            l = '{:.9f}'.format(round(float(dat[3]), 9))
                            h = '{:.3f}'.format(round(float(dat[4]), 3))
                            g = ''
                            if float(b) != 0 and float(l) != 0:
                                # 世界測地系(JGD2011）に変換
                                crsSrc = QgsCoordinateReferenceSystem(4326)
                                crsDest = QgsCoordinateReferenceSystem(EPSG2)
                                trans = QgsCoordinateTransform(crsSrc, crsDest)
                                y1,x1 = trans.transform(QgsPoint(float(l), float(b)))
                                x01 = '{:.7f}'.format(round(x1, 7))
                                y01 = '{:.7f}'.format(round(y1, 7))
                                # プロジェクトのCRSに変換
                                if self.ui.check_convert2.isChecked():
                                    if epsgcode == '':#カスタムCRSの場合
                                        Dest_crs = QgsCoordinateReferenceSystem()
                                        Dest_crs.createFromId(EPSG3, QgsCoordinateReferenceSystem.InternalCrsId)
                                        crsDest = QgsCoordinateReferenceSystem(Dest_crs)
                                    else:
                                        crsDest = QgsCoordinateReferenceSystem(EPSG3)
                                    trans = QgsCoordinateTransform(crsSrc, crsDest)
                                    y2,x2 = trans.transform(QgsPoint(float(l), float(b)))
                                    x02 = '{:.7f}'.format(round(x2, 7))
                                    y02 = '{:.7f}'.format(round(y2, 7))
                                else:
                                    x2 = ''
                                    y2 = ''
                                    x02 = ''
                                    y02 = ''
                                    EPSG3 = ''
                                log_line = daytime + ',' + b + ',' + l + ',' + h + ',' + g + ',' + str(EPSG2) + ',' + x01 + ',' + y01 + ',' + str(EPSG3) + ',' + x02 + ',' + y02 + ',,'
                                if not self.ui.epoch.isChecked():
                                    n = 0
                                    x001 = 0
                                    y001 = 0
                                    h001 = 0
                                ep = int(self.ui.lineEdit_epoch.text())
                                if self.ui.epoch.isChecked() and n != ep:
                                    x001 = x001 + x1
                                    y001 = y001 + y1
                                    h001 = h001 + float(h)
                                    n = n + 1
                                    self.ui.epoch.setText(str(n))
                                elif n == ep:
                                    x001 = x001 / ep
                                    y001 = y001 / ep
                                    h001 = h001 / ep
                                    self.ui.epoch.setChecked(False)
                                    self.ui.label_x2.setText(str('{:.3f}'.format(round(x001, 3))))
                                    self.ui.label_y2.setText(str('{:.3f}'.format(round(y001, 3))))
                                    self.ui.label_h2.setText(str('{:.3f}'.format(round(h001, 3))))
                                    self.ui.epoch.setText(u'開始')
                                    crsSrc = QgsCoordinateReferenceSystem(EPSG2)
                                    crsDest = QgsCoordinateReferenceSystem(4326)
                                    trans = QgsCoordinateTransform(crsSrc, crsDest)
                                    y3,x3 = trans.transform(QgsPoint(y001, x001))
                                    n = 0
                                    x001 = 0
                                    y001 = 0
                                    h001 = 0
                                    d = datetime.datetime.today()
                                    daystr = d.strftime("%Y%m%d")
                                    csvname = os.path.dirname(__file__) + '/log/' + daystr + '.csv'
                                    pn = self.ui.lineEdit_pn.text()
                                    csvtext = daytime + ',' + str(x3) + ',' + str(y3) + ',' + self.ui.label_h2.text() + ',,' + str(EPSG2) + ',' + self.ui.label_x2.text() + ',' + self.ui.label_y2.text() + ',,,,' + pn + ',\n'
                                    fname = codecs.open(csvname, 'a', 'shift-jis')
                                    fname.write(csvtext)
                                    fname.close()
                                    self.ui.lineEdit_pn.setFocus()
                                if self.ui.check_tansaku.isChecked():
                                    self.tansaku()
                                if self.ui.check_plot.isChecked():
                                    self.plot()
                                if self.ui.check_shousai.isChecked():
                                    try:
                                        self.ui.label_x.setText(str('{:.3f}'.format(round(x1, 3))))
                                        self.ui.label_y.setText(str('{:.3f}'.format(round(y1, 3))))
                                        self.ui.label_h.setText(h)
                                        self.ui.label_ratio.setText(dat[14])
                                    except:
                                        pass
                                if self.ui.check_log.isChecked():
                                    self.ui.textLog.append(log_line)
                                with open(os.path.dirname(__file__) + "/log/" + startTime + u"_" + log_name + u".log", "a") as myfile:
                                    myfile.write(log_line + '\n')
                                with open(os.path.dirname(__file__) + "/log/" + startTime + u"_" + log_name + u".pos", "a") as myfile:
                                    myfile.write(line + '\n')
                except:
                    pass
            ser.close()


    # 探索
    def tansaku(self):
        x2 = round(float(self.ui.x1.text()), 3)
        y2 = round(float(self.ui.y1.text()), 3)
        dx = x2 - x1
        dy = y2 - y1
        #点間距離
        s = math.sqrt(dx ** 2 + dy ** 2)
        #方向角
        RAD = 3.14159265358979 / 180
        pi = math.atan2(1, 1) * 4;
        if dx == 0:
            if dy == 0:
                a = 0
            elif dy > 0:
                a = pi / 2
            elif dy < 0:
                a = pi * 3 / 2
        elif dx != 0:
            ha = math.atan2(math.fabs(dy), math.fabs(dx))
            if dx > 0:
                if dy >= 0:
                    a = ha
                if dy <  0:
                    a = 2 * pi - ha
            if dx < 0:
                if dy >= 0:
                    a = pi - ha
                if dy <  0:
                    a = pi + ha
        if a >= 2 * pi:
            a = a - 2 * pi
        dd = a / RAD;
        self.ui.a.setText('{:.9f}'.format(round(dd, 9)) + u' ／ ' + '{:.3f}'.format(round(s, 3)))


    # プロット
    def plot(self):
        # プロジェクトのCRSを取得
        epsgGet()
        if epsgcode != '':
            code = epsgcode
        else:
            code = usercode
        layer = QgsVectorLayer('Point?crs=epsg:' + code + '&field=name&field=Latitude&field=Longitude&field=Altitude&field=Geoid&field=EPSG1&field=X1&field=Y1&field=EPSG2&field=X2&field=Y2', daytime, 'memory')
        if epsgcode != '':#プロジェクトのCRSがカスタムCRS以外の場合
            layer.setCrs(QgsCoordinateReferenceSystem(4326, QgsCoordinateReferenceSystem.EpsgCrsId))
        else:#プロジェクトのCRSがカスタムCRSの場合
            layer.setCrs(QgsCoordinateReferenceSystem(4326, QgsCoordinateReferenceSystem.InternalCrsId))
        provider = layer.dataProvider()
        feature = QgsFeature()
        geometry = QgsGeometry.fromPoint(QgsPoint(float(l), float(b)))
        feature.setGeometry(geometry)
        feature.setAttributes([daytime, l, b, h, g, EPSG2, x01, y01, EPSG3, x02, y02])
        provider.addFeatures([feature])
        QgsMapLayerRegistry.instance().addMapLayer(layer, False)
        if QgsProject.instance().layerTreeRoot().findGroup(self.tr('Log')) == None:
            QgsProject.instance().layerTreeRoot().insertChildNode(0,QgsLayerTreeGroup(self.tr('Log')))
        group = QgsProject.instance().layerTreeRoot().findGroup(self.tr('Log'))
        group.insertLayer(0,layer)
        if self.ui.check_centering.isChecked():
            crsSrc = QgsCoordinateReferenceSystem(4326)
            if code != '4326' or code != '4612':
                if epsgcode != '':
                    crsDest = QgsCoordinateReferenceSystem(int(code))
                else:
                    Dest_crs = QgsCoordinateReferenceSystem()
                    Dest_crs.createFromId(int(code), QgsCoordinateReferenceSystem.InternalCrsId)
                    crsDest = QgsCoordinateReferenceSystem(Dest_crs)
                trans = QgsCoordinateTransform(crsSrc, crsDest)
                y3,x3 = trans.transform(QgsPoint(float(l), float(b)))
                rect = QgsRectangle(float(y3),float(x3),float(y3),float(x3))
            else:
                rect = QgsRectangle(float(l),float(b),float(l),float(b))
            map.setExtent(rect)
        return


    # ×ボタンがクリックされた時
    def closeEvent(self,event):
        ws = QSettings("M'z Room", "GNSS_Logger_Plugin")
        ws.setValue("setting/window_w", self.width())   #ウィンドウの幅
        ws.setValue("setting/window_h", self.height())  #ウィンドウの高さ
        ws.setValue("setting/window_x", self.x() + 8)   #ウィンドウの横位置
        ws.setValue("setting/window_y", self.y() + 31)  #ウィンドウの縦位置
        ws.setValue("setting/GNSS_kei", self.ui.kei.currentIndex())                     # 平面直角座標系
        ws.setValue("setting/GNSS_Port", self.ui.comboBox_port.currentIndex())          # Port
        ws.setValue("setting/GNSS_Baudrate", self.ui.comboBox_baudrate.currentIndex())  # Baudrate
        ws.setValue("setting/GNSS_Sentence", self.ui.comboBox_sentence.currentIndex())  # Sentence
        if self.ui.radioButton_time1.isChecked():                                       # Time
            gnss_time = '1'
        else:
            gnss_time = '0'
        ws.setValue("setting/GNSS_Time", gnss_time)
        if self.ui.check_plot.isChecked():                                              # プロット
            gnss_plot = '1'
        else:
            gnss_plot = '0'
        ws.setValue("setting/GNSS_Plot", gnss_plot)
        if self.ui.check_centering.isChecked():                                         # センタリング
            gnss_centering = '1'
        else:
            gnss_centering = '0'
        ws.setValue("setting/GNSS_Centering", gnss_centering)
        if self.ui.check_log.isChecked():                                               # ログ表示
            gnss_log = '1'
        else:
            gnss_log = '0'
        ws.setValue("setting/GNSS_Log", gnss_log)
        if self.ui.check_shousai.isChecked():                                           # 詳細表示
            gnss_view = '1'
        else:
            gnss_view = '0'
        ws.setValue("setting/GNSS_View", gnss_view)


    # GPX保存
    def gpx_save(self):
        dialog = QFileDialog()
    	dialog.setDirectory(os.path.dirname(__file__) + u'/log/')
        # ログファイルを読み込む
        log_name = dialog.getOpenFileName(self, u'ログファイルを選択してください。', '', '*.log *.csv')
        if not log_name:
            return
        filename = os.path.basename(log_name)
        fn, ext = os.path.splitext(filename)
        for line in open(log_name, 'r'):
            if ext == 'log' and line[0:1] == '$':
                QMessageBox.information(self, u'エラー', u'「GNRMC/GPRMC」「GNGGA/GPGGA」「GNGLL/GPGLL」いずれかのログを選択してください。')
                return
        ws = QSettings("M'z Room", "GNSS_Logger_Plugin")
        reg_path = ws.value("setting/reg_path", os.path.dirname(__file__))
        os.chdir(reg_path.replace('/', '//'))
        gpxname = dialog.getSaveFileName(self, u'GPXで保存', fn + '.gpx', '*.gpx')
        if not gpxname:
            return
        reg_path = os.path.dirname(gpxname)
        ws.setValue("setting/reg_path", reg_path)
        gpx_text = '<?xml version=' + '"' + '1.0' + '"' + ' encoding=' + '"' + 'UTF-8' + '"' + '?>\n'
        gpx_text = gpx_text + '<gpx xmlns:xsi=' + '"' + 'http://www.w3.org/2001/XMLSchema-instance' + '"' + ' xmlns=' + '"' + 'http://www.topografix.com/GPX/1/0' + '"' +' xsi:schemaLocation=' + '"' + 'http://www.topografix.com/GPX/1/1 http://www.topografix.com/GPX/1/1/gpx.xsd' + '"' + '>\n'
        gpx_text = gpx_text + '    <trk>\n'
        gpx_text = gpx_text + '        <name>GNSS_Logger_Plugin</name>\n'
        gpx_text = gpx_text + '        <number>1</number>\n'
        gpx_text = gpx_text + '        <trkseg>\n'
        for line in open(log_name, 'r'):
            dat = line.replace((u'\n'), '').replace((u'\r'), '').split(',')
            name = str(dat[0])
            if len(name) == 14:
                day_time = name[0:4] + '-' + name[4:6] + '-' + name[6:8] + 'T' + name[8:10] + ':' + name[10:12] + ':' + name[12:14] + 'Z'
            elif len(name) == 15:
                day_time = '20' + name[4:6] + '-' + name[2:4] + '-' + name[0:2] + 'T' + name[6:8] + ':' + name[8:10] + ':' + name[10:15] + 'Z'
            elif len(name) == 9:
                day_time = 'T' + name[0:2] + ':' + name[2:4] + ':' + name[4:9] + 'Z'
            elif len(name) == 10:
                day_time = 'T' + name[0:2] + ':' + name[2:4] + ':' + name[4:10] + 'Z'
            bb = str(dat[1])
            ll = str(dat[2])
            hh = str(dat[3])
            gpx_text = gpx_text + '            <trkpt lat=' + '"' + bb + '"' + ' lon=' + '"' + ll + '"' + '><ele>' + hh + '</ele><time>' + day_time + '</time></trkpt>\n'
        gpx_text = gpx_text + '        </trkseg>\n'
        gpx_text = gpx_text + '    </trk>\n'
        gpx_text = gpx_text + '</gpx>\n'
        fname = open(gpxname, 'w')
        fname.write(gpx_text)
        fname.close()


    # SIMA保存
    def sima_save(self):
        dialog = QFileDialog()
    	dialog.setDirectory(os.path.dirname(__file__) + u'/log/')
        # ログファイルを読み込む
        log_name = dialog.getOpenFileName(self, u'ログファイルを選択してください。', '', '*.log *.csv')
        if not log_name:
            return
        filename = os.path.basename(log_name)
        fn, ext = os.path.splitext(filename)
        for line in open(log_name, 'r'):
            if ext == 'log' and line[0:1] == '$':
                QMessageBox.information(self, u'エラー', u'「GNRMC/GPRMC」「GNGGA/GPGGA」「GNGLL/GPGLL」いずれかのログを選択してください。')
                return
        ws = QSettings("M'z Room", "GNSS_Logger_Plugin")
        reg_path = ws.value("setting/reg_path", os.path.dirname(__file__))
        os.chdir(reg_path.replace('/', '//'))
        simaname = dialog.getSaveFileName(self, u'SIMAで保存', fn + '.sim', '*.sim')
        if not simaname:
            return
        reg_path = os.path.dirname(simaname)
        ws.setValue("setting/reg_path", reg_path)
        sima_text = 'G00,01,GNSS_Loggerﾃﾞｰﾀ,,\n'
        sima_text = sima_text + 'Z00,座標ﾃﾞｰﾀ,\n'
        sima_text = sima_text + 'A00,\n'
        k = 1
        for line in open(log_name, 'r'):
            dat = line.replace((u'\n'), '').replace((u'\r'), '').split(',')
            if dat[11] != '':
                name = str(dat[11])
            else:
                name = str(dat[0])
            hh = str(dat[3])
            xx = str(dat[6])
            yy = str(dat[7])
            sima_text = sima_text + 'A01,' + str(k) + ',' + name + ',' + xx + ',' + yy + ',' + hh + ',\n'
            k = k + 1
        sima_text = sima_text + 'A99,\n'
        fname = open(simaname, 'w')
        fname.write(sima_text)
        fname.close()


    # KML保存
    def kml_save(self):
        dialog = QFileDialog()
    	dialog.setDirectory(os.path.dirname(__file__) + u'/log/')
        # ログファイルを読み込む
        log_name = dialog.getOpenFileName(self, u'ログファイルを選択してください。', '', '*.log *.csv')
        if not log_name:
            return
        filename = os.path.basename(log_name)
        fn, ext = os.path.splitext(filename)
        for line in open(log_name, 'r'):
            if ext == 'log' and line[0:1] == '$':
                QMessageBox.information(self, u'エラー', u'「GNRMC/GPRMC」「GNGGA/GPGGA」「GNGLL/GPGLL」いずれかのログを選択してください。')
                return
        ws = QSettings("M'z Room", "GNSS_Logger_Plugin")
        reg_path = ws.value("setting/reg_path", os.path.dirname(__file__))
        os.chdir(reg_path.replace('/', '//'))
        kmlname = dialog.getSaveFileName(self, u'KMLで保存', fn + '.kml', '*.kml')
        if not kmlname:
            return
        reg_path = os.path.dirname(kmlname)
        ws.setValue("setting/reg_path", reg_path)
        point_text = ''
        rosen_text = ''
        kml_text = '<?xml version=' + '"' + '1.0' + '"' + ' encoding=' + '"' + 'UTF-8' + '"' + '?>\n'
        kml_text = kml_text + '<kml xmlns=' + '"' + 'http://www.opengis.net/kml/2.2' + '"' + ' xmlns:gx=' + '"' + 'http://www.google.com/kml/ext/2.2' + '"' +' xmlns:kml=' + '"' + 'http://www.opengis.net/kml/2.2' + '"' + ' xmlns:atom=' + '"' + 'http://www.w3.org/2005/Atom' + '"' + '>\n'
        kml_text = kml_text + '    <Document>\n'
        kml_text = kml_text + '        <name></name>\n'
        kml_text = kml_text + '        <StyleMap id=' + '"' + 'my_line' + '"' + '>\n'
        kml_text = kml_text + '            <Pair>\n'
        kml_text = kml_text + '                <key>normal</key>\n'
        kml_text = kml_text + '                <styleUrl>#s_my_line</styleUrl>\n'
        kml_text = kml_text + '            </Pair>\n'
        kml_text = kml_text + '        </StyleMap>\n'
        kml_text = kml_text + '        <Style id=' + '"' + 's_my_line' + '"' + '>\n'
        kml_text = kml_text + '            <LineStyle>\n'
        kml_text = kml_text + '                <color>880000ff</color>\n'
        kml_text = kml_text + '                <width>2</width>\n'
        kml_text = kml_text + '            </LineStyle>\n'
        kml_text = kml_text + '            <PolyStyle>\n'
        kml_text = kml_text + '                <fill>0</fill>\n'
        kml_text = kml_text + '            </PolyStyle>\n'
        kml_text = kml_text + '        </Style>\n'
        k = 0
        for line in open(log_name, 'r'):
            dat = line.replace((u'\n'), '').replace((u'\r'), '').split(',')
            if dat[11] != '':
                name = str(dat[11])
            else:
                name = str(dat[0])
            bb = str(dat[1])
            ll = str(dat[2])
            hh = str(dat[3])
            epsg = str(dat[5])
            xx = str(dat[6])
            yy = str(dat[7])
            point_text = point_text + '        <Placemark>\n'
            point_text = point_text + '          <name>' + name + '</name>\n'
            point_text = point_text + '          <description><![CDATA[<div>EPSG:' + epsg + '</div><div>X= ' + xx + '</div><div>Y= ' + yy + '</div><div>H= ' + hh + '</div>]]></description>\n'
            point_text = point_text + '          <Point>\n'
            point_text = point_text + '              <coordinates>' + ll + ',' + bb + ',' + hh + '</coordinates>\n'
            point_text = point_text + '          </Point>\n'
            point_text = point_text + '        </Placemark>\n'
            point = ll + ',' + bb + ',' + hh + ' '
            if k == 0:
                rosen_text = rosen_text + '        <Placemark>\n'
                rosen_text = rosen_text + '            <name>track</name>\n'
                rosen_text = rosen_text + '            <description>track</description>\n'
                rosen_text = rosen_text + '            <styleUrl>#my_line</styleUrl>\n'
                rosen_text = rosen_text + '            <LineString>\n'
                rosen_text = rosen_text + '                <tessellate>1</tessellate>\n'
                rosen_text = rosen_text + '                <altitudeMode>relativeToGround</altitudeMode>\n'
                rosen_text = rosen_text + '                    <coordinates>\n'
                coordinates = '                            ' + point
                k = 1
            else:
                coordinates = coordinates + point
        rosen_text = rosen_text + coordinates +'\n'
        rosen_text = rosen_text + '                    </coordinates>\n'
        rosen_text = rosen_text + '            </LineString>\n'
        rosen_text = rosen_text + '        </Placemark>\n'
        kml_text = kml_text + rosen_text + point_text
        kml_text = kml_text + '    </Document>\n'
        kml_text = kml_text + '</kml>\n'
        fname2 = open(kmlname, 'w')
        fname2.write(kml_text)
        fname2.close()


    # プロジェクトのEPSG変更
    def change_CRS(self):
        dat = self.ui.comboBox_CRS.currentText().split(':')
        epsg = int(dat[1])
        custom = str(QgsCoordinateReferenceSystem(int(epsg)).authid())
        if custom != '':#指定のCRSがカスタムCRSでない場合
            new_crs = QgsCoordinateReferenceSystem(epsg, QgsCoordinateReferenceSystem.EpsgCrsId)
        else:
            new_crs = QgsCoordinateReferenceSystem(int(epsg), QgsCoordinateReferenceSystem.InternalCrsId)
        map.setDestinationCrs(new_crs)
        map.setMapUnits(0)
        map.refresh()


# プロジェクトのEPSG取得
def epsgGet():
  global epsgcode
  global usercode
  crs = qgis.utils.iface.mapCanvas().mapRenderer().destinationCrs()
  epsgcode = str(crs.authid())
  if epsgcode[0:5] == 'EPSG:':
    epsgcode = epsgcode.replace('EPSG:', '')
    usercode = ''
  else:
    usercode = epsgcode.replace('USER:', '')
    epsgcode = ""
